// commands, macros and stuff.

use crate::{errors::TexError, Res};

#[derive(Debug, Clone)]
pub struct Command {
    pub name: String,
    pub nargs: usize,
    pub def: String, // actual latex, cannot help
}

// impl AsLatex for Command {
//     fn to_string(&self) -> String {
//         // format!(
//         //     "\\newcommand{{\\{}}}[{}]{{{}}} ",
//         //     self.name, self.nargs, self.def
//         // )
//         if args.len() != self.nargs {
//             return Err(TexError::ArgLen.into());
//         }
//         let mut temp = args
//         .iter()
//         .map(|x| format!("\\{}{{{x}}}", self.name))
//         .collect::<String>();
//     }
// }

impl Command {
    pub fn new(name: &str, nargs: usize, def: &str) -> Self {
        Self {
            name: name.to_string(),
            nargs,
            def: def.to_string(),
        }
    }

    pub fn call(&self, args: Vec<&str>) -> Res<String> {
        if args.len() != self.nargs {
            return Err(TexError::ArgLen.into());
        }
        let temp = format!(
            "\\{}{}",
            self.name,
            args.iter().map(|x| format!("{{{x}}}")).collect::<String>()
        );
        Ok(temp)
    }

    pub fn declare(&self) -> String {
        format!(
            "\\newcommand{{\\{}}}[{}]{{{}}} ",
            self.name, self.nargs, self.def
        )
    }
}
